@@ -22,7 +22,6 @@ Kenozooid's logbook command line user interface.
 """
 
 import sys
-import itertools
 import os.path
 from functools import partial
 from lxml import etree as et
@@ -30,7 +29,7 @@ import logging
 
 from kenozooid.component import inject
 from kenozooid.cli import CLIModule, ArgumentError, add_master_command, \
-        _dive_data
+    add_uddf_input
 from kenozooid.component import query, params
 from kenozooid.util import nformat
 
@@ -76,11 +75,10 @@ class ListDives(object):
         """
         import kenozooid.logbook as kl
 
-        files = args.input
-
-        for fin in files:
+        for fin in args.input:
+            dives = kl.find_dives([fin])
             print('{}:'.format(fin))
-            for i, d in enumerate(kl.list_dives(fin), 1):
+            for i, d in enumerate(kl.list_dives(dives), 1):
                 print('{:5}: {:>9} {:>9} ({:>5}) {:>9} {:>9}'.format(i,
                     d[0], d[1], d[2] or ' --- ', d[3], d[4]))
 
@@ -114,62 +112,47 @@ class AddDive(object):
         Execute command for adding dives into logbook file.
         """
         import kenozooid.logbook as kl
+        import kenozooid.data as kd
         from dateutil.parser import parse as dparse
 
         lfile = args.logbook
 
         datetime = dparse(args.datetime)
         depth = float(args.depth)
-        duration = float(args.duration)
+        duration = float(args.duration) * 60
 
         site = args.site
         buddy = args.buddy
 
-        kl.add_dive(lfile, datetime, depth, duration, qsite=site,
-                qbuddies=buddy)
+        dive = kd.Dive(datetime=datetime, depth=depth, duration=duration)
+        kl.add_dive(dive, lfile, qsite=site, qbuddies=buddy)
 
 
 
 @inject(CLIModule, name='dive copy')
 class CopyDive(object):
     """
-    Copy dive to logbook file.
+    Copy dives to logbook file.
     """
-    description = 'copy dive to logbook file'
+    description = 'copy dives to logbook file'
 
     @classmethod
     def add_arguments(self, parser):
         """
         Add options for dive copying to logbook file.
         """
-        parser.add_argument('dive', help='dive number in input file')
-        parser.add_argument('input', help='input file with dive data')
-        parser.add_argument('-s', '--site', metavar='site', help='dive site')
-        parser.add_argument('-b', '--buddy',
-                nargs='+',
-                metavar='buddy',
-                help='dive buddies')
+        add_uddf_input(parser)
         parser.add_argument('logbook', help='logbook file')
 
 
     def __call__(self, args):
         """
-        Execute command for adding dives into logbook file.
+        Execute command for copying dives into logbook file.
         """
         import kenozooid.logbook as kl
-        from dateutil.parser import parse as dparse
 
-        dive_no = args.dive
-        ifile = args.input
-        lfile = args.logbook
-
-        dive_no = int(dive_no)
-
-        site = args.site
-        buddy = args.buddy
-
-        kl.add_dive(lfile, dive_no=dive_no, pfile=ifile, qsite=site,
-                qbuddies=buddy)
+        r, f = args.input
+        kl.copy_dives(f, r, args.logbook)
 
 
 
