@@ -60,17 +60,50 @@ UDDF_PROFILE = b"""\
         </divecomputer>
       </equipment>
     </owner>
+    <buddy id="b1"><personal>
+        <firstname>F1 AA</firstname><lastname>L1 X1</lastname>
+        <membership memberid="m1" organisation="CFT"/>
+    </personal></buddy>
+    <buddy id="b2"><personal>
+        <firstname>F2 BB</firstname><lastname>L2 Y2</lastname>
+        <membership memberid="m2" organisation="CFT"/>
+    </personal></buddy>
   </diver>
+  <divesite>
+      <site id='markgraf'><name>SMS Markgraf</name><geography><location>Scapa Flow</location></geography></site>
+      <site id='konig'><name>SMS Konig</name><geography><location>Scapa Flow</location></geography></site>
+  </divesite>
+  <gasdefinitions>
+    <mix id="air">
+      <name>Air</name>
+      <o2>21</o2>
+      <he>0</he>
+    </mix>
+    <mix id="ean39">
+      <name>EAN39</name>
+      <o2>39</o2>
+      <he>0</he>
+    </mix>
+    <mix id="tx1248">
+      <name>TX 12/48</name>
+      <o2>12</o2>
+      <he>48</he>
+    </mix>
+  </gasdefinitions>
   <profiledata>
     <repetitiongroup>
+      <!-- this dive has to have gas switching -->
       <dive id='d01'>
         <informationbeforedive>
             <datetime>2009-09-19T13:10:23</datetime>
+            <link ref='konig'/>
+            <link ref='b1'/>
         </informationbeforedive>
         <samples>
           <waypoint>
             <depth>1.48</depth>
             <divetime>0</divetime>
+            <switchmix ref='air'/>
             <temperature>289.02</temperature>
             <divemode type='opencircuit'/>
           </waypoint>
@@ -82,6 +115,7 @@ UDDF_PROFILE = b"""\
           <waypoint>
             <depth>3.58</depth>
             <divetime>20</divetime>
+            <switchmix ref='ean39'/>
           </waypoint>
         </samples>
         <informationafterdive>
@@ -91,9 +125,12 @@ UDDF_PROFILE = b"""\
             <averagedepth>10.1</averagedepth>
         </informationafterdive>
       </dive>
+      <!-- this dive shall have not gas switching -->
       <dive id='d02'>
         <informationbeforedive>
             <datetime>2010-10-30T13:24:43</datetime>
+            <link ref='b1'/>
+            <link ref='b2'/>
         </informationbeforedive>
         <samples>
           <waypoint>
@@ -122,6 +159,37 @@ UDDF_PROFILE = b"""\
             <lowesttemperature>250.4</lowesttemperature>
         </informationafterdive>
       </dive>
+      <dive id='d03'>
+        <informationbeforedive>
+            <datetime>2011-11-11T11:11:11</datetime>
+        </informationbeforedive>
+        <samples>
+          <waypoint>
+            <depth>2.61</depth>
+            <divetime>0</divetime>
+            <temperature>296.73</temperature>
+            <divemode type='closedcircuit'/>
+          </waypoint>
+          <waypoint>
+            <depth>4.18</depth>
+            <divetime>10</divetime>
+          </waypoint>
+          <waypoint>
+            <depth>6.25</depth>
+            <divetime>20</divetime>
+          </waypoint>
+          <waypoint>
+            <depth>8.32</depth>
+            <divetime>29</divetime>
+            <temperature>297.26</temperature>
+          </waypoint>
+        </samples>
+        <informationafterdive>
+            <diveduration>29</diveduration>
+            <greatestdepth>30.2</greatestdepth>
+            <lowesttemperature>251.4</lowesttemperature>
+        </informationafterdive>
+      </dive>
     </repetitiongroup>
   </profiledata>
 </uddf>
@@ -251,9 +319,10 @@ class FindDataTestCase(unittest.TestCase):
         """
         f = BytesIO(UDDF_PROFILE)
         depths = list(ku.find(f, '//uddf:waypoint//uddf:depth/text()'))
-        self.assertEqual(7, len(depths))
+        self.assertEqual(11, len(depths))
 
-        expected = ['1.48', '2.43', '3.58', '2.61', '4.18', '6.25', '8.32']
+        expected = ['1.48', '2.43', '3.58', '2.61', '4.18', '6.25', '8.32',
+                '2.61', '4.18', '6.25', '8.32',]
         self.assertEqual(expected, depths)
 
 
@@ -270,6 +339,7 @@ class FindDataTestCase(unittest.TestCase):
         self.assertEquals(251.4, dive.temp)
         self.assertEquals(10.1, dive.avg_depth)
         self.assertEquals('opencircuit', dive.mode)
+        self.assertEquals(3, len(list(dive.profile)))
 
 
     def test_profile_data(self):
@@ -1026,35 +1096,64 @@ class PostprocessingTestCase(unittest.TestCase):
 
 
 
-class NodeRangeTestCase(unittest.TestCase):
+class RangeTestCase(unittest.TestCase):
     """
-    Node range tests.
+    Number range tests.
     """
     def test_simple(self):
         """
-        Test parsing simple numerical ranges
+        Test parsing simple number ranges
         """
-        self.assertEquals('1 <= position() and position() <= 3',
-                ku.node_range('1-3'))
-        self.assertEquals('position() = 2 or position() = 4',
-                ku.node_range('2, 4'))
-        self.assertEquals('position() = 1 or position() = 3'
-                ' or 4 <= position() and position() <= 7',
-            ku.node_range('1,3,4-7'))
-        self.assertEquals('1 <= position()', ku.node_range('1-'))
-        self.assertEquals('position() <= 10', ku.node_range('-10'))
+        self.assertEquals('1 <= n and n <= 3', ku.parse_range('1-3'))
+        self.assertEquals('n == 2 or n == 4', ku.parse_range('2, 4'))
+        self.assertEquals('n == 1 or n == 3 or 4 <= n and n <= 7',
+            ku.parse_range('1,3,4-7'))
+        self.assertEquals('1 <= n', ku.parse_range('1-'))
+        self.assertEquals('n <= 10', ku.parse_range('-10'))
 
 
     def test_errors(self):
         """
-        Test invalid ranges
+        Test invalid number ranges
         """
-        self.assertRaises(ku.RangeError, ku.node_range, '30--')
-        self.assertRaises(ku.RangeError, ku.node_range, '30-2-')
-        self.assertRaises(ku.RangeError, ku.node_range, '1,a,2')
-        self.assertRaises(ku.RangeError, ku.node_range, '1-a,3')
+        self.assertRaises(ku.RangeError, ku.parse_range, '30--')
+        self.assertRaises(ku.RangeError, ku.parse_range, '30-2-')
+        self.assertRaises(ku.RangeError, ku.parse_range, '1,a,2')
+        self.assertRaises(ku.RangeError, ku.parse_range, '1-a,3')
 
 
+    def test_in_range_query_all(self):
+        """
+        Test XPath number range query for all nodes
+        """
+        f = BytesIO(UDDF_PROFILE)
+        q = '//uddf:dive[in-range(position(), $nodes)]/@id'
+        ids = list(ku.find(f, ku.XPath(q), nodes=None))
+        self.assertEquals(['d01', 'd02', 'd03'], ids)
+
+
+    def test_in_range_query(self):
+        """
+        Test XPath number range query for some nodes
+        """
+        f = BytesIO(UDDF_PROFILE)
+
+        q = '//uddf:dive[in-range(position(), "2")]/@id'
+        dt = list(ku.find(f, ku.XPath(q)))
+        self.assertEquals(['d02'], dt)
+
+        q = '//uddf:dive[in-range(position(), "2-3")]/@id'
+        dt = list(ku.find(f, ku.XPath(q)))
+        self.assertEquals(['d02', 'd03'], dt)
+
+        q = '//uddf:dive[in-range(position(), "2,3")]/@id'
+        dt = list(ku.find(f, ku.XPath(q)))
+        self.assertEquals(['d02', 'd03'], dt)
+
+        q = '//uddf:dive[in-range(position(), "2-")]/@id'
+        dt = list(ku.find(f, ku.XPath(q)))
+        self.assertEquals(['d02', 'd03'], dt)
+
 
 class NodeCopyTestCase(unittest.TestCase):
     """
@@ -1088,7 +1187,8 @@ class NodeCopyTestCase(unittest.TestCase):
         t = et.XML(TARGET)
         c = ku.xp_first(t, '//uddf:c')
 
-        r = ku.copy(a, c)
+        with ku.NodeCopier(t) as nc:
+            r = nc.copy(a, c)
         self.assertEquals('{http://www.streit.cc/uddf/3.1/}a1', r.tag)
 
         n = ku.xp_first(t, '//uddf:a1')
@@ -1096,6 +1196,46 @@ class NodeCopyTestCase(unittest.TestCase):
         self.assertEquals(2, len(n))
 
 
+    def test_duplicate_copy(self):
+        """
+        Test duplicate UDDF node copy
+        """
+        SOURCE = b"""\
+<?xml version="1.0" encoding="utf-8"?>
+<uddf xmlns="http://www.streit.cc/uddf/3.1/" version="3.1.0">
+    <a1 id = 'id_1'>
+        <a2/>
+        <a2/>
+    </a1>
+    <b1>
+        <b2/>
+    </b1>
+</uddf>
+"""
+        TARGET = b"""\
+<?xml version="1.0" encoding="utf-8"?>
+<uddf xmlns="http://www.streit.cc/uddf/3.1/" version="3.1.0">
+    <c/>
+</uddf>
+"""
+        s = BytesIO(SOURCE)
+        a, *_ = ku.find(s, '//uddf:a1')
+
+        t = et.XML(TARGET)
+        c = ku.xp_first(t, '//uddf:c')
+
+        with ku.NodeCopier(t) as nc:
+            r = nc.copy(a, c)
+            self.assertEquals('{http://www.streit.cc/uddf/3.1/}a1', r.tag)
+
+            # copy once again the same node
+            self.assertTrue(nc.copy(a, c) is None)
+
+        n = ku.xp_first(t, '//uddf:a1')
+        self.assertTrue(n is not None)
+        self.assertEquals(2, len(n))
+
+
     def test_copy_ref_descendant_nodes(self):
         """
         Test copying UDDF node with descendants referencing themselves
@@ -1124,7 +1264,8 @@ class NodeCopyTestCase(unittest.TestCase):
         t = et.XML(TARGET)
         c = ku.xp_first(t, '//uddf:c')
 
-        ku.copy(a, c)
+        with ku.NodeCopier(t) as nc:
+            nc.copy(a, c)
 
         n = ku.xp_first(t, '//uddf:a1')
         self.assertTrue(n is not None)
@@ -1162,7 +1303,8 @@ class NodeCopyTestCase(unittest.TestCase):
         t = et.XML(TARGET)
         c = ku.xp_first(t, '//uddf:c')
 
-        ku.copy(a, c)
+        with ku.NodeCopier(t) as nc:
+            nc.copy(a, c)
         sd = et.tostring(t, pretty_print=True)
 
         n = ku.xp_first(t, '//uddf:a1')
@@ -1201,7 +1343,8 @@ class NodeCopyTestCase(unittest.TestCase):
         t = et.XML(TARGET)
         c = ku.xp_first(t, '//uddf:c')
 
-        ku.copy(a, c)
+        with ku.NodeCopier(t) as nc:
+            nc.copy(a, c)
 
         n = ku.xp_first(t, '//uddf:a1')
         self.assertTrue(n is not None)
@@ -1239,7 +1382,8 @@ class NodeCopyTestCase(unittest.TestCase):
         t = et.XML(TARGET)
         c = ku.xp_first(t, '//uddf:c')
 
-        ku.copy(a, c)
+        with ku.NodeCopier(t) as nc:
+            nc.copy(a, c)
         sd = et.tostring(t, pretty_print=True)
 
         n = ku.xp_first(t, '//uddf:a1')
@@ -1282,7 +1426,8 @@ class NodeCopyTestCase(unittest.TestCase):
         t = et.XML(TARGET)
         c = ku.xp_first(t, '//uddf:c')
 
-        ku.copy(a, c)
+        with ku.NodeCopier(t) as nc:
+            nc.copy(a, c)
         sd = et.tostring(t, pretty_print=True)
 
         # a2 nodes are removed
@@ -1330,7 +1475,8 @@ class NodeCopyTestCase(unittest.TestCase):
         t = et.XML(TARGET)
         c = ku.xp_first(t, '//uddf:c')
 
-        ku.copy(a, c)
+        with ku.NodeCopier(t) as nc:
+            nc.copy(a, c)
         sd = et.tostring(t, pretty_print=True)
 
         # a2 nodes are removed
@@ -1372,10 +1518,45 @@ class NodeCopyTestCase(unittest.TestCase):
         t = et.XML(TARGET)
         c = ku.xp_first(t, '//uddf:c')
 
-        self.assertRaises(ValueError, ku.copy, a, c)
+        with ku.NodeCopier(t) as nc:
+            self.assertRaises(ValueError, nc.copy, a, c)
         self.assertTrue(ku.xp_first(t, '//uddf:c') is not None)
 
 
+    def test_copy_no_overwrite(self):
+        """
+        Test no overwriting when copying UDDF node 
+        """
+        SOURCE = b"""\
+<?xml version="1.0" encoding="utf-8"?>
+<uddf xmlns="http://www.streit.cc/uddf/3.1/" version="3.1.0">
+    <a1>
+        <a2 id = 'id-a1'/>
+        <a2 id = 'id-a2'/>
+    </a1>
+</uddf>
+"""
+        TARGET = b"""\
+<?xml version="1.0" encoding="utf-8"?>
+<uddf xmlns="http://www.streit.cc/uddf/3.1/" version="3.1.0">
+    <c>
+        <a3 id = 'id-a1'/>
+    </c>
+</uddf>
+"""
+        s = BytesIO(SOURCE)
+        a, *_ = ku.find(s, '//uddf:a2[@id="id-a1"]')
+
+        t = et.XML(TARGET)
+        c = ku.xp_first(t, '//uddf:c')
+
+        with ku.NodeCopier(t) as nc:
+            nc.copy(a, c)
+
+        ids = tuple(ku.xp(c, 'descendant-or-self::uddf:*/@id'))
+        self.assertEquals(('id-a1',), ids)
+
+
 
 class UDDFSaveTestCase(unittest.TestCase):
     def setUp(self):
