@@ -102,6 +102,7 @@ XP_DEFAULT_DIVE_DATA = (
     XPath('uddf:informationafterdive/uddf:lowesttemperature/text()'),
     XPath('uddf:informationafterdive/uddf:averagedepth/text()'),
     XPath('uddf:samples/uddf:waypoint/uddf:divemode[1]/@type'),
+    None,
 )
 
 # XPath queries for default dive profile sample data
@@ -175,13 +176,23 @@ XP_FIND_SITE = XPath('/uddf:uddf/uddf:divesite/uddf:site[' \
     ' or contains(uddf:geography/uddf:location/text(), $site)' \
     ']')
 
+# XPath query to find dives
+XP_FIND_DIVES = XPath('/uddf:uddf/uddf:profiledata' \
+    '/uddf:repetitiongroup/uddf:dive[in-range(position(), $nodes)]')
+
+# XPath query to find dive gases
+XP_FIND_DIVE_GASES = XPath('/uddf:uddf/uddf:gasdefinitions' \
+    '/uddf:mix[@id=/uddf:uddf/uddf:profiledata/uddf:repetitiongroup' \
+    '/uddf:dive[in-range(position(), $nodes)]' \
+    '/uddf:samples/uddf:waypoint/uddf:switchmix/@ref]')
+
 
 class RangeError(ValueError):
     """
     Error raised when a range cannot be parsed.
 
     .. seealso::
-        node_range
+        parse_range
     """
     pass
 
@@ -373,9 +384,10 @@ def dive_data(node, fields=None, queries=None, parsers=None):
     .. seealso:: :py:func:`find_data`
     """
     if fields is None:
-        fields = ('datetime', 'depth', 'duration', 'temp', 'avg_depth', 'mode')
+        fields = ('datetime', 'depth', 'duration', 'temp', 'avg_depth',
+                'mode', 'profile')
         queries = XP_DEFAULT_DIVE_DATA
-        parsers = (dparse, float, float, float, float, str)
+        parsers = (dparse, float, float, float, float, str, dive_profile)
 
     return find_data('Dive', node, fields, queries, parsers)
 
@@ -537,22 +549,22 @@ def site_data(node, fields=None, queries=None, parsers=None):
     return find_data('DiveSite', node, fields, queries, parsers)
 
 
-def node_range(s):
+def parse_range(s):
     """
-    Parse textual representation of number range into XPath expression.
+    Parse textual representation of number range into Python expression.
 
     Examples of a ranges
 
-    >>> node_range('1-3,5')
-    '1 <= position() and position() <= 3 or position() = 5'
+    >>> parse_range('1-3,5')
+    '1 <= n and n <= 3 or n == 5'
 
-    >>> node_range('-3,10')
-    'position() <= 3 or position() = 10'
+    >>> parse_range('-3,10')
+    'n <= 3 or n == 10'
 
     Example of infinite range
 
-    >>> node_range('20-')
-    '20 <= position()'
+    >>> parse_range('20-')
+    '20 <= n'
 
     :Parameters:
      s
@@ -563,17 +575,16 @@ def node_range(s):
         for r in s.split(','):
             d = r.split('-')
             if len(d) == 1:
-                data.append('position() = %d' % int(d[0]))
+                data.append('n == %d' % int(d[0]))
             elif len(d) == 2:
                 p1 = d[0].strip()
                 p2 = d[1].strip()
                 if p1 and p2:
-                    data.append('%d <= position() and position() <= %d' \
-                            % (int(p1), int(p2)))
+                    data.append('{} <= n and n <= {}'.format(int(p1), int(p2)))
                 elif p1 and not p2:
-                    data.append('%d <= position()' % int(p1))
+                    data.append('{} <= n'.format(int(p1)))
                 elif not p1 and p2:
-                    data.append('position() <= %d' % int(p2))
+                    data.append('n <= {}'.format(int(p2)))
             else:
                 raise RangeError('Invalid range %s' % s)
     except ValueError as ex:
@@ -581,6 +592,34 @@ def node_range(s):
     return ' or '.join(data)
 
 
+def in_range(ctx, pos, nodes):
+    """
+    XPath expression function to restrict position of a node to be within
+    numeric range.
+
+    :Parameters:
+     ctx
+        XPath context object.
+     pos
+        Node position.
+     nodes
+        Number range, i.e. "2-3".
+
+    .. seealso:: :py:func:`parse_range`
+    """
+    if not nodes:
+        return True
+    if 'in-range' not in ctx.eval_context:
+        nr = parse_range(nodes)
+        fstr = 'ctx.eval_context["in-range"] = lambda n: {}'.format(nr)
+        exec(fstr)
+    return ctx.eval_context['in-range'](pos)
+
+# register in-range XPath function
+ns = et.FunctionNamespace(None)
+ns['in-range'] = in_range
+
+
 def _field(node, query, parser):
     """
     Find text value of a node starting from specified XML node.
@@ -597,7 +636,7 @@ def _field(node, query, parser):
      parser
         Parser to convert text value to requested type.
     """
-    data = query(node)
+    data = [node] if query is None else query(node)
     if data:
         return parser(data[0])
 
@@ -606,9 +645,12 @@ def _record(rt, node, queries, parsers):
     """
     Create record with data.
 
-    The record data is found with XPath expressions objects starting from
-    XML node.  The data is converted to their appropriate type using
-    parsers.
+    The record data is found with queries (XPath expressions objects)
+    starting from XML node.
+    
+    The data is converted to their appropriate type using parsers.
+
+    If query is `None`, then record data is node itself.
 
     :Parameters:
      rt
@@ -1209,57 +1251,97 @@ def reorder(doc):
         rg.append(n)
 
 
-def copy(node, target):
+class NodeCopier(object):
     """
-    Copy node from UDDF document to target node in destination UDDF
-    document. Target node becomes parent of node to be copied.
+    UDDF dcument node copier.
 
-    The copying works under following assumptions
+    See :py:func:`NodeCopier.copy` for details.
 
-    - whole node is being copied including its descendants
-    - if copied nodes reference non-descendant nodes and they do _not_
-      exist in destination document, then referencing nodes are _removed_
-    - if, due to node removal, its parent node becomes empty, then parent
-      is removed, too
-
-    Copy of the node is returned.
-
-    :Parameters:
-     node
-        Node to copy.
-     target
-        The future parent of the node to be copied.
+    :Attributes:
+     doc
+        The target document.
+     doc_ids
+        The cache of target document ids.
     """
-    cn = deepcopy(node)
+    def __init__(self, doc):
+        """
+        Initialize node copier.
+        """
+        self.doc = doc
+        self.doc_ids = set(xp(doc, '//uddf:*/@id'))
 
-    # get all ids
-    s1 = set(xp(target, '//uddf:*/@id'))
-    s2 = set(xp(cn, 'descendant-or-self::uddf:*/@id'))
-    ids = s1.union(s2)
 
-    # get referencing nodes
-    nodes = list(xp(cn, 'descendant-or-self::uddf:*[@ref]'))
-    refs = set(k.get('ref') for k in nodes)
+    def __enter__(self):
+        """
+        Create UDDF node copier context manager.
+        """
+        return self
 
-    left = refs - ids
-    log.debug('references to remove: {} = {} - {}'.format(left, refs, ids))
 
-    if cn.get('ref') in left:
-        raise ValueError('Node to copy references non-existing node')
+    def __exit__(self, *args):
+        """
+        Close UDDF node copier context manager.
+        """
+        pass
 
-    # remove referencing nodes to missing data
-    to_remove = (n for n in nodes if n.get('ref') in left)
-    assert cn.getparent() is None
-    for n in to_remove:
-        p = n.getparent()
-        while p is not None and len(p) == 1:
-            n = p
+
+    def copy(self, node, target):
+        """
+        Copy node from UDDF document to target node in destination UDDF
+        document. Target node becomes parent of node to be copied.
+
+        The copying works under following assumptions
+
+        - whole node is being copied including its descendants
+        - node is not copied if it has id and id already exists in the target
+        - if copied nodes reference non-descendant nodes and they do _not_
+          exist in destination document, then referencing nodes are _removed_
+        - if, due to node removal, its parent node becomes empty, then parent
+          is removed, too
+
+        Copy of the node is returned.
+
+        :Parameters:
+         node
+            Node to copy.
+         target
+            The future parent of the copied node.
+        """
+        cn = deepcopy(node)
+
+        cn_id = cn.get('id')
+        if cn_id in self.doc_ids:
+            log.debug('id {} already exists, not copying'.format(cn_id))
+            return None
+
+        s_ids = set(xp(cn, 'descendant-or-self::uddf:*/@id'))
+        self.doc_ids.update(s_ids)
+
+        # get referencing nodes
+        nodes = list(xp(cn, 'descendant-or-self::uddf:*[@ref]'))
+        refs = set(k.get('ref') for k in nodes)
+
+        left = refs - self.doc_ids
+        if __debug__:
+            log.debug('references to remove: {} = {} - {}'.format(left,
+                refs, self.doc_ids))
+
+        if cn.get('ref') in left:
+            raise ValueError('Node to copy references non-existing node')
+
+        # remove nodes referencing missing data
+        to_remove = (n for n in nodes if n.get('ref') in left)
+        assert cn.getparent() is None
+        for n in to_remove:
             p = n.getparent()
-        if p is not None:
-            p.remove(n)
+            while p is not None and len(p) == 1:
+                n = p
+                p = n.getparent()
+            if p is not None:
+                p.remove(n)
 
-    target.append(cn)
-    return cn
+        target.append(cn)
+        return cn
 
 
 def _set_id(node):
